local function SpawnMagicSparkles(mo, sprkcolor, sparkx, sparky, sparkz, overlay, random)
	local spark = P_SpawnMobj(sparkx, sparky, sparkz, MT_THOK)
	if overlay
		spark.type = MT_OVERLAY
		spark.target = mo
		spark.spriteyoffset = P_RandomRange(-1, mo.height/FRACUNIT)*FRACUNIT
		spark.spritexoffset = P_RandomRange(-mo.radius/FRACUNIT, mo.radius/FRACUNIT)*FRACUNIT
	end
	if random
		local randomx = mo.x+FixedMul(P_RandomRange(-25,25)*FRACUNIT,sin(P_RandomRange(0,360)*ANG1))
		local randomy = mo.y+FixedMul(P_RandomRange(-25,25)*FRACUNIT,cos(P_RandomRange(0,360)*ANG1))
		local randomz = mo.z + P_RandomRange(-1, mo.height/FRACUNIT)*FRACUNIT
		P_TeleportMove(spark, randomx, randomy, randomz)
	end
	spark.state = S_MAGICSPARK1
	spark.color = sprkcolor
	if mo.eflags & MFE_VERTICALFLIP
		spark.eflags = $|MFE_VERTICALFLIP
		spark.flags2 = $|MF2_OBJECTFLIP
	else
		spark.eflags = $ & ~MFE_VERTICALFLIP
		spark.flags2 = $ & ~MF2_OBJECTFLIP
	end
	return spark
end

rawset(_G, "M_UseMana", function(player, much)
	if player.valid and player.magicmana
	and player.magicmana >= much
	and not (player.magicmana - much <= -1)
		player.magicmana = $ - much
	end
end)

local magiccolors = {SKINCOLOR_GREEN,SKINCOLOR_RED,SKINCOLOR_CYAN,SKINCOLOR_BLUE,SKINCOLOR_WHITE,SKINCOLOR_ORANGE,SKINCOLOR_YELLOW,SKINCOLOR_VAPOR}
local nonecolors = {SKINCOLOR_GREEN,SKINCOLOR_FOREST,SKINCOLOR_EMERALD,SKINCOLOR_SEAFOAM,SKINCOLOR_APPLE}
local firecolors = {SKINCOLOR_SALMON,SKINCOLOR_RED,SKINCOLOR_RUBY,SKINCOLOR_KETCHUP,SKINCOLOR_CRIMSON,SKINCOLOR_FLAME}
local icecolors = {SKINCOLOR_AZURE,SKINCOLOR_SKY,SKINCOLOR_SLATE,SKINCOLOR_BLUEBELL,SKINCOLOR_TEAL,SKINCOLOR_CYAN}
local watercolors = {SKINCOLOR_BLUE,SKINCOLOR_COBALT,SKINCOLOR_ICY,SKINCOLOR_CERULEAN,SKINCOLOR_CORNFLOWER,SKINCOLOR_AQUA}
local windcolors = {SKINCOLOR_WHITE,SKINCOLOR_AETHER,SKINCOLOR_GREY,SKINCOLOR_CARBON,SKINCOLOR_SILVER,SKINCOLOR_SLATE}
local bombcolors = {SKINCOLOR_SANDY,SKINCOLOR_ORANGE,SKINCOLOR_COPPER,SKINCOLOR_RUST,SKINCOLOR_SUNSET,SKINCOLOR_APRICOT}
local magnetcolors = {SKINCOLOR_YELLOW,SKINCOLOR_GOLD,SKINCOLOR_OLIVE,SKINCOLOR_PERIDOT,SKINCOLOR_APPLE,SKINCOLOR_LIME}
local eleccolors = {SKINCOLOR_ROSY,SKINCOLOR_PURPLE,SKINCOLOR_MAGENTA,SKINCOLOR_VAPOR,SKINCOLOR_NEON,SKINCOLOR_VIOLET}

local msshield = {
[0] = S_MSSHIELD_F1,
[1] = S_MSSHIELD_F2}

local function SpawnMagicPentogram(mo, magicolor, addx, addy, addz, splat, magicfuse, mrollangle)
	local pent = P_SpawnMobjFromMobj(mo, addx, addy, addz, MT_THOK)
	pent.fuse = magicfuse
	pent.scale = 0
	pent.target = mo
	pent.destscale = FRACUNIT
	pent.state = S_MAGICPENTOGRAM1
	pent.color = magicolor
	pent.rollangle = mrollangle
	pent.tics = magicfuse
	
	local overlay = P_SpawnMobjFromMobj(mo, addx, addy, addz, MT_OVERLAY)
	overlay.fuse = magicfuse
	overlay.state = S_MAGICPENTOGRAM2
	overlay.color = magicolor
	overlay.target = pent
	overlay.scale = 0
	overlay.destscale = FRACUNIT
	overlay.rollangle = -mrollangle
	overlay.tics = magicfuse
	if splat
		pent.flags2 = $|MF2_SPLAT
		pent.renderflags = $|RF_FLOORSPRITE
		pent.angle = pent.rollangle
		pent.type = MT_OVERLAY
		pent.target = mo
		
		if overlay
			overlay.flags2 = $|MF2_SPLAT
			overlay.renderflags = $|RF_FLOORSPRITE
			overlay.angle = overlay.rollangle
		end	
	else
		pent.angle = mo.angle+ANGLE_90
		pent.spriteyoffset = addz
		pent.type = MT_THOK
		if overlay
			overlay.angle = mo.angle
			overlay.spriteyoffset = addz
		end
	end
end

local function P_SpawnIcePlatform(mo, whex, whey, whez)
	local ice = P_SpawnMobj(whex, whey, whez, MT_ICEPLATFORMMAGIC)
end

local numtostring = {
	[1] = "Fireball",
	[2] = "Ice platform",
	[3] = "Attack boost",
	[4] = "Defense boost",
	[5] = "Add Rings",
	[6] = "Lose Rings",
	[7] = "Tornado",
	[8] = "Fire-dash",
	[9] = "Fire-jump",
	[10] = "Spindash",
	[11] = "Fast regen.",
	[12] = "Triple jump",
	[0] = "None"
}

local stringtonum = {
	["Fireball"] = 1,
	["Ice platform"] = 2,
	["Attack boost"] = 3,
	["Defense boost"] = 4,
	["Add Rings"] = 5,
	["Lose Rings"] = 6,
	["Tornado"] = 7,
	["Fire-dash"] = 8,
	["Fire-jump"] = 9,
	["Spindash"] = 10,
	["Fast regen."] = 11,
	["Triple jump"] = 12,
	["None"] = 0
}

local MENUCAP = 50

addHook("PlayerSpawn", function(player)
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and player.powers[pw_carry] != CR_NIGHTSMODE
		player.pullout = 0
		player.pullnum = 0
		player.pullin = false
		player.regainmana = 0
		player.magicmana = 255
		player.cantshowmagic = 0
		player.tripjump = 1
		player.atkboost = 0
		player.defboost = 0
	end
	
	if player.valid and player.playerstate == PST_LIVE
	and player.powers[pw_carry] != CR_NIGHTSMODE
		player.atkboost = 0
		player.defboost = 0
		player.magicmana = 255
	end
end)	

addHook("PlayerThink", function(player)
	if player.valid and player.playerstate == PST_LIVE
	and player.powers[pw_carry] != CR_NIGHTSMODE
		player.atkboost = $ or 0
		player.defboost = $ or 0
		if player.atkboost
			player.atkboost = $ - 1
			P_SpawnAura(player, SKINCOLOR_RED, FRACUNIT/4, true)
		end
		if player.defboost
			player.defboost = $ - 1
			P_SpawnAura(player, SKINCOLOR_COBALT, FRACUNIT/3, true)
		end
	end
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and player.powers[pw_carry] != CR_NIGHTSMODE
		player.magicpentroll = $ or 0
		player.pullout = $ or 0
		player.pullnum = $ or 0
		player.mainmagic = $ or "Fireball"
		player.passmagic = $ or "None"
		player.magicpentroll = $ + 3*ANG1
		player.regainmana = $ or 0
		player.magicmana = $ or 0
		player.cantshowmagic = $ or 0
		player.tripjump = $ or 0
		if player.powers[pw_super] >= TICRATE
			player.magicmana = $ + 1
		end
		if P_IsObjectOnGround(player.mo)
			player.tripjump = 1
		end
		if player.cantshowmagic
			player.cantshowmagic = $ - 1
			player.culdntshow = true
		else
			if player.culdntshow
				player.culdntshow = false
				player.cus3cant = false
				player.cus2cant = false
			end
		end
		if stringtonum[player.mainmagic] == 10
			player.charability2 = CA2_SPINDASH
		else
			player.charability2 = 0
		end
		if player.passmagic and stringtonum[player.passmagic] == 8
		and player.dashmode > 3*TICRATE and player.magicmana
		and player.speed >= 15*FRACUNIT and P_IsObjectOnGround(player.mo)
			P_ElementalFire(player, false)
			if (leveltime%5) == 1
				M_UseMana(player, 1)
			end	
		end
		if player.passmagic and stringtonum[player.passmagic] == 11
		and player.speed <= 15*FRACUNIT and P_IsObjectOnGround(player.mo)
			if (leveltime%5) == 1
				player.magicmana = $ + 1
			end	
		end
		if player.magicmana > 255
			player.magicmana = 255
		end
		if player.magicmana < 0
			player.magicmana = 0
		end
		if player.speed <= 15*FRACUNIT
			player.regainmana = $ + 1
			if (player.regainmana%TICRATE) == 1
				if player.magicmana < 255
					player.magicmana = $ + 5
				end
			end
		end
		/*if player.passmagic and stringtonum[player.passmagic] == 2
			local addx = FixedMul(250*FRACUNIT, cos(player.mo.angle))
			local addy = FixedMul(250*FRACUNIT, sin(player.mo.angle))
			local show = P_SpawnMobjFromMobj(player.mo, addx, addy, 0, MT_THOK)
			show.state = S_ICEPLATFORMMAGIC
			show.fuse = 3
		end*/
		if player.mainmagic
		and stringtonum[player.mainmagic] == 3
			player.target = F_SearchForPlayers(player)
			if player.target and player.target.valid
				P_SpawnLockOn(player, player.target, S_LOCKON1)
			end
		end
		if player.mainmagic
		and stringtonum[player.mainmagic] == 4
			player.target = F_SearchForPlayers(player)
			if player.target and player.target.valid
				P_SpawnLockOn(player, player.target, S_LOCKON3)
			end
		end
		if player.mainmagic
		and stringtonum[player.mainmagic] == 5
			player.target = F_SearchForPlayers(player)
			if player.target and player.target.valid
				P_SpawnLockOn(player, player.target, S_LOCKON4)
			end
		end
		if player.mainmagic
		and stringtonum[player.mainmagic] == 6
			player.target = F_SearchForPlayers(player)
			if player.target and player.target.valid
			and player.target.player and player.target.player.cmd.buttons & BT_TOSSFLAG
				P_SpawnLockOn(player, player.target, S_LOCKON2)
			end
		end
		if player.cmd.buttons & BT_SPIN
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 3
			and player.magicmana >= 10
				M_UseMana(player, 10)
				if player.target and player.target.valid
				and player.target.player and not player.target.player.atkboost
					player.target.player.atkboost = TICRATE*30
					local addx = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local addy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					SpawnMagicPentogram(player.mo, SKINCOLOR_RED, addx, addy, player.mo.height/2, false, 15, player.magicpentroll)
					S_StartSound(player.target, sfx_supert)
				elseif not player.atkboost
					player.atkboost = TICRATE*30
					SpawnMagicPentogram(player.mo, SKINCOLOR_RED, 0, 0, FRACUNIT/15, true, 2, player.magicpentroll)
					S_StartSound(player.mo, sfx_supert)
				end
			end
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 4
			and player.magicmana >= 10
				M_UseMana(player, 25)
				if player.target and player.target.valid
				and player.target.player and not player.target.player.defboost
					player.target.player.defboost = TICRATE*30
					local addx = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local addy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					SpawnMagicPentogram(player.mo, SKINCOLOR_COBALT, addx, addy, player.mo.height/2, false, 15, player.magicpentroll)
					S_StartSound(player.target, sfx_shield)
				elseif not player.defboost
					player.defboost = TICRATE*30
					SpawnMagicPentogram(player.mo, SKINCOLOR_COBALT, 0, 0, FRACUNIT/15, true, 2, player.magicpentroll)
					S_StartSound(player.mo, sfx_shield)
				end
				
			end
			
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 5
			and player.magicmana >= 1
			and player.rings and player.rings >= 10
				if player.target and player.target.valid
				and player.target.player
					player.target.player.rings = $ + 10
					player.rings = $ - 10
					S_StartSound(player.target, sfx_itemup)
					P_SpawnMobjFromMobj(player.target, 0, 0, player.target.height, MT_RING_ICON)
					M_UseMana(player, 1)
					local addx = FixedMul(50*FRACUNIT, cos(player.mo.angle))
					local addy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
					SpawnMagicPentogram(player.mo, SKINCOLOR_GOLD, addx, addy, player.mo.height/2, false, 15, player.magicpentroll)
				end
			end
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 6
			and player.magicmana >= 1
				if player.target and player.target.valid
				and player.target.player and player.target.player.rings
				and player.target.player.rings >= 10
				and player.target.player.cmd.buttons & BT_TOSSFLAG
					player.target.player.rings = $ - 10
					player.rings = $ + 10
					M_UseMana(player, 1)
					S_StartSound(player.target, sfx_antiri)
					P_SpawnMobjFromMobj(player.mo, 0, 0, player.target.height, MT_RING_ICON)
					SpawnMagicPentogram(player.mo, SKINCOLOR_GOLD, 0, 0, FRACUNIT/15, true, 2, player.magicpentroll)
				end
			end
		
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 1
			and player.magicmana >= 10
				M_UseMana(player, 10)
				P_SPMAngle(player.mo, MT_FIREBALL, player.mo.angle)
				local addx = FixedMul(50*FRACUNIT, cos(player.mo.angle))
				local addy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
				SpawnMagicPentogram(player.mo, SKINCOLOR_FLAME, addx, addy, player.mo.height/2, false, 15, player.magicpentroll)
				S_StartSound(player.mo, sfx_mario7)
			end
			
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 2
			and player.magicmana >= 50
				M_UseMana(player, 50)
				local addx = FixedMul(50*FRACUNIT, cos(player.mo.angle))
				local addy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
				SpawnMagicPentogram(player.mo, SKINCOLOR_ICY, addx, addy, player.mo.height/2, false, 15, player.magicpentroll)
				local addx2 = FixedMul(250*FRACUNIT, cos(player.mo.angle))
				local addy2 = FixedMul(250*FRACUNIT, sin(player.mo.angle))
				
				local ice = P_SpawnMobjFromMobj(player.mo, 0, 0, player.mo.height/2, MT_DEMONFIRE)
				ice.target = player.mo
				ice.flags = $ & ~(MF_NOGRAVITY)
				ice.wherx = player.mo.x + addx2
				ice.whery = player.mo.y + addy2
				ice.wherz = player.mo.z + 50*FRACUNIT
				ice.platformball = true
				ice.state = S_BIGFIREBAR1
				ice.colorized = true
				ice.scale = FRACUNIT/3
				ice.color = SKINCOLOR_ICY
				ice.returnto = TICRATE
				P_ParabolicMove(ice, player.mo.x + addx2, player.mo.y + addy2, player.mo.z + 50*FRACUNIT, 16*FRACUNIT)
				S_StartSound(player.mo, sfx_iceb)
				//P_SpawnIcePlatform(player.mo, player.mo.x + addx2, player.mo.y + addy2, player.mo.z)
			end
			
			if not player.spin
			and player.mainmagic
			and stringtonum[player.mainmagic] == 7
			and player.magicmana >= 85
				M_UseMana(player, 85)
				local tornado = P_SPMAngle(player.mo, MT_TORNADO, player.mo.angle)
				local addx = FixedMul(50*FRACUNIT, cos(player.mo.angle))
				local addy = FixedMul(50*FRACUNIT, sin(player.mo.angle))
				SpawnMagicPentogram(player.mo, SKINCOLOR_AETHER, addx, addy, player.mo.height/2, false, 15, player.magicpentroll)
				S_StartSound(player.mo, sfx_wdjump)
			end
			
			player.spin = true
		else
			player.spin = false
		end
		if player.pullout
			player.pullnum = $ + 1
		end
		if player.pullout and player.pullout < (100-MENUCAP)
		and not player.pullin
			player.pullout = $ + player.pullnum
		end
		if player.pullout
		and player.pullout > (100-MENUCAP)
			player.pullout = (100-MENUCAP)
		end
		if player.pullout
		and player.pullin
			player.pullout = $ - player.pullnum
		end
		if not player.pullout
			if player.pullin
				player.pullin = false
			end	
		end
		if player.pullout < 0
			player.pullout = 0
		end
		if player.pullout and player.pullout >= (100-MENUCAP)
			if player.cmd.buttons & BT_CUSTOM3
				if not player.cus3
					player.mainmagic = numtostring[stringtonum[player.mainmagic]+1]
					player.cantshowmagic = 3
					player.cus3cant = true
				end
				player.cus3 = true
			else
				player.cus3 = false
			end
			if player.cmd.buttons & BT_CUSTOM2
				if not player.cus2
					player.passmagic = numtostring[stringtonum[player.passmagic]+1]
					player.cantshowmagic = 3
					player.cus2cant = true
				end
				player.cus2 = true
			else
				player.cus2 = false
			end
		end
		if player.mainmagic
			if stringtonum[player.mainmagic] >= 8
			and stringtonum[player.mainmagic] <= 9
				player.mainmagic = numtostring[10]
			end
			if stringtonum[player.mainmagic] > 10
				player.mainmagic = numtostring[1]
			end
		end
		if player.passmagic
			if stringtonum[player.passmagic] > 9
			and stringtonum[player.passmagic] < 11
				player.passmagic = numtostring[11]
			end
			if stringtonum[player.passmagic] > 12
				player.passmagic = numtostring[0]
			end
			if stringtonum[player.passmagic] < 8
			and stringtonum[player.passmagic] > 0
				player.passmagic = numtostring[8]
			end
		end
		if player.cmd.buttons & BT_CUSTOM1
			if not player.cus1
			and not player.pullout
				player.pullout = 1
				player.pullnum = 1
			end
			if not player.cus1
			and player.pullout
			and player.pullout >= (100-MENUCAP)
				player.pullin = true
				player.pullnum = 1
			end
			player.cus1 = true
		else
			player.cus1 = false
		end
		P_SpawnAura(player, SKINCOLOR_MAGICALMETAL, 1, false)
		if player.followmobj and player.speed
		and (player.mo.state != S_PLAY_ROLL and player.mo.state != S_PLAY_JUMP)
		and player.followmobj.state != S_INVISIBLE
		and player.followmobj.state != S_NULL
			if (leveltime&4) == 0
				local sparks = SpawnMagicSparkles(player.followmobj, P_RandomRange(1, #skincolors-1), 0, 0, 0, false, true)
				sparks.momx = P_RandomRange(-5, 5)*FRACUNIT
				sparks.momy = P_RandomRange(-5, 5)*FRACUNIT
				sparks.momz = P_RandomRange(-2, 1)*FRACUNIT
			end	
		end
		if player.metaljump
			player.metaljump = $ - 1
			//SpawnMagicPentogram(player.mo, SKINCOLOR_MAGICALMETAL, 0, 0, FRACUNIT/15, true, 2, player.magicpentroll)
			P_SetObjectMomZ(player.mo, FRACUNIT, true)
			if player.passmagic and stringtonum[player.passmagic] == 9
			and player.magicmana
				for i = 1, 12
					local ranx = FixedMul(P_RandomRange(0, 25)*FRACUNIT, cos(P_RandomRange(0, 360)*ANG1))
					local rany = FixedMul(P_RandomRange(0, 25)*FRACUNIT, sin(P_RandomRange(0, 360)*ANG1))
					local ranz = P_RandomRange(0, player.mo.height/FRACUNIT)*FRACUNIT
					local fire = P_SpawnMobjFromMobj(player.mo, ranx, rany, ranz, MT_DEMONFIRE)
					fire.target = player.mo
					fire.colorized = true
					fire.fuse = 15
					fire.state = S_FLAMEJETFLAME1
					fire.flags = $ & ~MF_NOGRAVITY
					fire.destscale = -1
				end
				player.powers[pw_invulnerability] = $ + 1
				if (player.metaljump%2) == 1
					M_UseMana(player, 1)
				end	
			end
			if player.passmagic and stringtonum[player.passmagic] == 12
			and not player.tripjump
			and (player.pflags & PF_THOKKED)
				SpawnMagicPentogram(player.mo, SKINCOLOR_MAGICALMETAL, 0, 0, FRACUNIT/15, true, 2, player.magicpentroll)
			end
			if (leveltime%5) == 0
				local ghost = P_SpawnGhostMobj(player.mo)
				ghost.destscale = 0
				ghost.color = SKINCOLOR_SKY
				ghost.colorized = true
			end	
			if player.mo.momz == 0
			and player.metaljump > 2
				P_SetObjectMomZ(player.mo, 5*FRACUNIT)
			end
			if player.metaljump == 1
				player.mo.momz = $/2
			end
			if player.metaljump < 20
				player.mo.state = S_PLAY_SPRING
			end
		end
		if player.cmd.buttons & BT_JUMP
			player.jump = true
		else
			player.jump = false
		end
	end
end)

addHook("AbilitySpecial", function(player)
	if player.valid and player.playerstate == PST_LIVE
	and player.mo.skin == "magicmetal" and not (player.powers[pw_carry] == CR_NIGHTSMODE)
		if not (player.pflags & PF_THOKKED)
		and not player.metaljump
			player.metaljump = 25
			P_SetObjectMomZ(player.mo, -5*FRACUNIT, true)
			local ghost = P_SpawnGhostMobj(player.mo)
			ghost.destscale = player.mo.scale + FRACUNIT/2
			ghost.colorized = true
			ghost.color = SKINCOLOR_BLACK
			ghost.state = S_BOOSTCIRCLE
			S_StartSound(player.mo, sfx_mswarp)
			player.mo.state = S_PLAY_CLIMB
			player.pflags = $|PF_JUMPED
			if player.passmagic and stringtonum[player.passmagic] == 12
			and player.magicmana and player.magicmana >= 10
				if player.tripjump
					player.tripjump = $ - 1
				else
					player.pflags = $|PF_THOKKED
					M_UseMana(player, 10)
				end
			else
				player.pflags = $|PF_THOKKED
			end
			if player.passmagic and stringtonum[player.passmagic] == 9
				player.pflags = $ & ~PF_NOJUMPDAMAGE
			else
				player.pflags = $|PF_NOJUMPDAMAGE
			end
		end
	end
end)

local stringtopatch = {
	["Fireball"] = "FIRMAGIC",
	["Ice platform"] = "ICEMAGIC",
	["Attack boost"] = "ATKMAGIC",
	["Defense boost"] = "DEFMAGIC",
	["Add Rings"] = "ADDRINGS",
	["Lose Rings"] = "MINRINGS",
	["Tornado"] = "WINDMAGIC",
	["Fire-dash"] = "FIRDSHMAGIC",
	["Fire-jump"] = "DBJUMPMAGIC",
	["Spindash"] = "SPNDSHNOTMAGIC",
	["Fast regen."] = "FASTREGENMAGIC",
	["Triple jump"] = "TPJUMPMAGIC",
	["None"] = "NOMAGIC"
}

hud.add(function(v, player)
	if player.valid and player.powers[pw_carry] != CR_NIGHTSMODE
	and player.mo.skin == "magicmetal"
		local menuy = 110
		local manay = 95
		if player.pullout
			v.draw(10 + player.pullout, menuy, v.cachePatch("MAGICSMOLMENU"), V_SNAPTOLEFT|V_HUDTRANS)
		else
			v.draw(10, menuy, v.cachePatch("MAGICSMOLMENU"), V_SNAPTOLEFT|V_HUDTRANS)
		end
		v.draw(0, manay, v.cachePatch("MANABAR"), V_SNAPTOLEFT|V_HUDTRANS)
		local color = SKINCOLOR_CRIMSON
		if player.magicmana
			if player.magicmana <= 25
				color = SKINCOLOR_CRIMSON
			elseif player.magicmana <= 50
				color = SKINCOLOR_SALMON
			elseif player.magicmana <= 100
				color = SKINCOLOR_COBALT
			elseif player.magicmana <= 150
				color = SKINCOLOR_BLUE	
			elseif player.magicmana <= 200
				color = SKINCOLOR_FOREST
			elseif player.magicmana <= 255
				color = SKINCOLOR_GREEN	
			end
		else
			color = SKINCOLOR_CRIMSON
		end
		if player.magicmana
			v.drawStretched(20*FRACUNIT + FRACUNIT, manay*FRACUNIT + FRACUNIT*3, (player.magicmana+1)/6*FRACUNIT, FRACUNIT, v.cachePatch("MANASEG"), V_SNAPTOLEFT|V_HUDTRANS, v.getColormap("sonic", color))
		end
		v.drawString(56, manay+4, "/ 255", V_SNAPTOLEFT|V_HUDTRANSHALF|V_GRAYMAP, "small-thin-right")
		if player.magicmana
			v.drawString(23, manay, player.magicmana, V_SNAPTOLEFT|V_HUDTRANS, "left")
		else
			v.drawString(23, manay, "0", V_SNAPTOLEFT|V_HUDTRANS|V_REDMAP, "left")
		end
		v.draw(10, menuy + 15, v.cachePatch("MAINMAGIC"), V_SNAPTOLEFT|V_HUDTRANS) //Main Magic frame
		v.draw(10, menuy + 47, v.cachePatch("PASSMAGIC"), V_SNAPTOLEFT|V_HUDTRANS) //Passive Magic frame
		if player.pullout
			v.drawString(8, menuy + 27, "Close Menu:", V_SNAPTOLEFT|V_HUDTRANSHALF|V_GRAYMAP|V_ALLOWLOWERCASE, "small-thin-center")
			if not player.cantshowmagic
				v.drawString(20, menuy + 16, string.sub("Choose main magic: C3", 0, player.pullout/2), V_SNAPTOLEFT|V_HUDTRANSHALF|V_GRAYMAP|V_ALLOWLOWERCASE, "small-thin")
				v.drawString(20, menuy + 49, string.sub("Choose passive magic: C2", 0, player.pullout/2), V_SNAPTOLEFT|V_HUDTRANSHALF|V_GRAYMAP|V_ALLOWLOWERCASE, "small-thin")
			end	
		else
			v.drawString(8, menuy + 27, "Open Menu:", V_SNAPTOLEFT|V_HUDTRANSHALF|V_GRAYMAP|V_ALLOWLOWERCASE, "small-thin-center")
		end	
		v.drawString(8, menuy + 32, "C1", V_SNAPTOLEFT|V_HUDTRANSHALF|V_GRAYMAP|V_ALLOWLOWERCASE, "small-thin-center")
		if not player.cantshowmagic
			if player.mainmagic
				if stringtopatch[player.mainmagic]
					v.draw(10, menuy + 15, v.cachePatch(stringtopatch[player.mainmagic]), V_SNAPTOLEFT|V_HUDTRANS) //Icon
				else
					v.draw(10, menuy + 15, v.cachePatch("UNKNOWNMAGIC"), V_SNAPTOLEFT|V_HUDTRANS)
				end
				if player.pullout
					v.drawString(20, menuy + 8, string.sub(player.mainmagic, 0, player.pullout/3), V_SNAPTOLEFT|V_HUDTRANS|V_ALLOWLOWERCASE, "thin") //Main magic text
				end	
			else
				v.draw(10, menuy + 15, v.cachePatch("NOMAGIC"), V_SNAPTOLEFT|V_HUDTRANS) //No magic
			end
			if player.passmagic
				if stringtopatch[player.passmagic]
					v.draw(10, menuy + 47, v.cachePatch(stringtopatch[player.passmagic]), V_SNAPTOLEFT|V_HUDTRANS) //Icon
				else
					v.draw(10, menuy + 47, v.cachePatch("UNKNOWNMAGIC"), V_SNAPTOLEFT|V_HUDTRANS)
				end	
				if player.pullout
					v.drawString(20, menuy + 41, string.sub(player.passmagic, 0, player.pullout/3), V_SNAPTOLEFT|V_HUDTRANS|V_ALLOWLOWERCASE, "thin") //Main magic text
				end	
			else
				v.draw(10, menuy + 47, v.cachePatch("NOMAGIC"), V_SNAPTOLEFT|V_HUDTRANS) //No magic
			end
		else
			v.draw(10, menuy + 15, v.cachePatch("LOADINGMAGIC"), V_SNAPTOLEFT|V_HUDTRANS)
			v.draw(10, menuy + 47, v.cachePatch("LOADINGMAGIC"), V_SNAPTOLEFT|V_HUDTRANS)
		end	
	end
end)